@@ -252,7 +252,7 @@ void KSudoku::startGame(const Game& game) {
 	widget->show();
 	widget->setFocus();
 
-	connect(game.interface(), SIGNAL(completed(bool,const QTime&,bool)), SLOT(onCompleted(bool,const QTime&,bool)));
+	connect(game.interface(), SIGNAL(completed(bool,QTime,bool)), SLOT(onCompleted(bool,QTime,bool)));
 	connect(game.interface(), SIGNAL(modified(bool)), SLOT(onModified(bool)));
 
 	adaptActions2View();
@@ -841,7 +841,7 @@ void KSudoku::optionsPreferences()
 	dialog->addPage(new KGameThemeSelector(dialog, Settings::self(), KGameThemeSelector::NewStuffDisableDownload), i18n("Theme"), "games-config-theme");
 
     dialog->setHelp(QString(),"ksudoku");
-	connect(dialog, SIGNAL(settingsChanged(const QString&)), SLOT(updateSettings()));
+	connect(dialog, SIGNAL(settingsChanged(QString)), SLOT(updateSettings()));
 	dialog->show();
 }
 
